@@ -23,6 +23,6 @@
 #include <Rinternals.h>
 
 SEXP git2r_revwalk_contributions(SEXP repo, SEXP topological, SEXP time, SEXP reverse);
-SEXP git2r_revwalk_list(SEXP repo, SEXP topological, SEXP time, SEXP reverse);
+SEXP git2r_revwalk_list(SEXP repo, SEXP topological, SEXP time, SEXP reverse, SEXP max_n);
 
 #endif
